package javax.swing.text;

import java.util.EventListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.EventListenerList;
import java.util.Vector;
import java.util.NoSuchElementException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Iterator;
import javax.swing.SwingUtilities;
import java.awt.Toolkit;
import java.awt.FontMetrics;
import javax.swing.plaf.FontUIResource;
import sun.font.FontUtilities;
import java.awt.Color;
import javax.swing.event.ChangeListener;
import java.util.Enumeration;
import java.util.Collections;
import java.util.WeakHashMap;
import java.lang.ref.WeakReference;
import java.util.Map;
import java.awt.Font;
import java.util.Hashtable;
import java.io.Serializable;

public class StyleContext implements Serializable, AbstractDocument.AttributeContext
{
    private static StyleContext defaultContext;
    public static final String DEFAULT_STYLE = "default";
    private static Hashtable<Object, String> freezeKeyMap;
    private static Hashtable<String, Object> thawKeyMap;
    private Style styles;
    private transient StyleContext.FontKey fontSearch;
    private transient Hashtable<StyleContext.FontKey, Font> fontTable;
    private transient Map<StyleContext.SmallAttributeSet, WeakReference<StyleContext.SmallAttributeSet>> attributesPool;
    private transient MutableAttributeSet search;
    private int unusedSets;
    static final int THRESHOLD = 9;
    
    public static final StyleContext getDefaultStyleContext() {
        if (StyleContext.defaultContext == null) {
            StyleContext.defaultContext = new StyleContext();
        }
        return StyleContext.defaultContext;
    }
    
    public StyleContext() {
        this.fontSearch = new StyleContext.FontKey(null, 0, 0);
        this.fontTable = new Hashtable();
        this.attributesPool = Collections.synchronizedMap(new WeakHashMap());
        this.search = new SimpleAttributeSet();
        this.styles = new StyleContext.NamedStyle(null);
        this.addStyle("default", null);
    }
    
    public Style addStyle(String s, Style style) {
        StyleContext.NamedStyle namedStyle = new StyleContext.NamedStyle(s, style);
        if (s != null) {
            this.styles.addAttribute(s, namedStyle);
            return namedStyle;
        }
        return namedStyle;
    }
    
    public void removeStyle(String s) {
        this.styles.removeAttribute(s);
    }
    
    public Style getStyle(String s) {
        return (Style)this.styles.getAttribute(s);
    }
    
    public Enumeration<?> getStyleNames() {
        return this.styles.getAttributeNames();
    }
    
    public void addChangeListener(ChangeListener changeListener) {
        this.styles.addChangeListener(changeListener);
    }
    
    public void removeChangeListener(ChangeListener changeListener) {
        this.styles.removeChangeListener(changeListener);
    }
    
    public ChangeListener[] getChangeListeners() {
        return ((StyleContext.NamedStyle)this.styles).getChangeListeners();
    }
    
    public Font getFont(AttributeSet set) {
        int n = 0;
        if (StyleConstants.isBold(set)) {
            n |= 0x1;
        }
        if (StyleConstants.isItalic(set)) {
            n |= 0x2;
        }
        String fontFamily = StyleConstants.getFontFamily(set);
        int fontSize = StyleConstants.getFontSize(set);
        if (StyleConstants.isSuperscript(set) || StyleConstants.isSubscript(set)) {
            fontSize -= 2;
        }
        return this.getFont(fontFamily, n, fontSize);
    }
    
    public Color getForeground(AttributeSet set) {
        return StyleConstants.getForeground(set);
    }
    
    public Color getBackground(AttributeSet set) {
        return StyleConstants.getBackground(set);
    }
    
    public Font getFont(String s, int n, int size) {
        this.fontSearch.setValue(s, n, size);
        FontUIResource value = (Font)this.fontTable.get(this.fontSearch);
        if (value == null) {
            Style style = this.getStyle("default");
            if (style != null) {
                Font font = (Font)style.getAttribute("FONT_ATTRIBUTE_KEY");
                if (font != null && font.getFamily().equalsIgnoreCase(s)) {
                    value = font.deriveFont(n, (float)size);
                }
            }
            if (value == null) {
                value = new Font(s, n, size);
            }
            if (!FontUtilities.fontSupportsDefaultEncoding(value)) {
                value = FontUtilities.getCompositeFontUIResource(value);
            }
            this.fontTable.put(new StyleContext.FontKey(s, n, size), value);
            return value;
        }
        else {
            return value;
        }
    }
    
    public FontMetrics getFontMetrics(Font font) {
        return Toolkit.getDefaultToolkit().getFontMetrics(font);
    }
    
    @Override
    public synchronized AttributeSet addAttribute(AttributeSet set, Object o, Object o2) {
        if (set.getAttributeCount() + 1 <= this.getCompressionThreshold()) {
            this.search.removeAttributes(this.search);
            this.search.addAttributes(set);
            this.search.addAttribute(o, o2);
            this.reclaim(set);
            return this.getImmutableUniqueSet();
        }
        MutableAttributeSet mutableAttributeSet = this.getMutableAttributeSet(set);
        mutableAttributeSet.addAttribute(o, o2);
        return mutableAttributeSet;
    }
    
    @Override
    public synchronized AttributeSet addAttributes(AttributeSet set, AttributeSet set2) {
        if (set.getAttributeCount() + set2.getAttributeCount() <= this.getCompressionThreshold()) {
            this.search.removeAttributes(this.search);
            this.search.addAttributes(set);
            this.search.addAttributes(set2);
            this.reclaim(set);
            return this.getImmutableUniqueSet();
        }
        MutableAttributeSet mutableAttributeSet = this.getMutableAttributeSet(set);
        mutableAttributeSet.addAttributes(set2);
        return mutableAttributeSet;
    }
    
    @Override
    public synchronized AttributeSet removeAttribute(AttributeSet set, Object o) {
        if (set.getAttributeCount() - 1 <= this.getCompressionThreshold()) {
            this.search.removeAttributes(this.search);
            this.search.addAttributes(set);
            this.search.removeAttribute(o);
            this.reclaim(set);
            return this.getImmutableUniqueSet();
        }
        MutableAttributeSet mutableAttributeSet = this.getMutableAttributeSet(set);
        mutableAttributeSet.removeAttribute(o);
        return mutableAttributeSet;
    }
    
    @Override
    public synchronized AttributeSet removeAttributes(AttributeSet set, Enumeration<?> enumeration) {
        if (set.getAttributeCount() <= this.getCompressionThreshold()) {
            this.search.removeAttributes(this.search);
            this.search.addAttributes(set);
            this.search.removeAttributes(enumeration);
            this.reclaim(set);
            return this.getImmutableUniqueSet();
        }
        MutableAttributeSet mutableAttributeSet = this.getMutableAttributeSet(set);
        mutableAttributeSet.removeAttributes(enumeration);
        return mutableAttributeSet;
    }
    
    @Override
    public synchronized AttributeSet removeAttributes(AttributeSet set, AttributeSet set2) {
        if (set.getAttributeCount() <= this.getCompressionThreshold()) {
            this.search.removeAttributes(this.search);
            this.search.addAttributes(set);
            this.search.removeAttributes(set2);
            this.reclaim(set);
            return this.getImmutableUniqueSet();
        }
        MutableAttributeSet mutableAttributeSet = this.getMutableAttributeSet(set);
        mutableAttributeSet.removeAttributes(set2);
        return mutableAttributeSet;
    }
    
    @Override
    public AttributeSet getEmptySet() {
        return SimpleAttributeSet.EMPTY;
    }
    
    @Override
    public void reclaim(AttributeSet set) {
        if (SwingUtilities.isEventDispatchThread()) {
            this.attributesPool.size();
        }
    }
    
    protected int getCompressionThreshold() {
        return 9;
    }
    
    protected StyleContext.SmallAttributeSet createSmallAttributeSet(AttributeSet set) {
        return new StyleContext.SmallAttributeSet(set);
    }
    
    protected MutableAttributeSet createLargeAttributeSet(AttributeSet set) {
        return new SimpleAttributeSet(set);
    }
    
    synchronized void removeUnusedSets() {
        this.attributesPool.size();
    }
    
    AttributeSet getImmutableUniqueSet() {
        StyleContext.SmallAttributeSet smallAttributeSet = this.createSmallAttributeSet(this.search);
        WeakReference<Object> weakReference = (WeakReference)this.attributesPool.get(smallAttributeSet);
        StyleContext.SmallAttributeSet referent;
        if (weakReference == null || (referent = (StyleContext.SmallAttributeSet)weakReference.get()) == null) {
            referent = smallAttributeSet;
            this.attributesPool.put(referent, new WeakReference(referent));
            return referent;
        }
        return referent;
    }
    
    MutableAttributeSet getMutableAttributeSet(AttributeSet set) {
        if (set instanceof MutableAttributeSet && set != SimpleAttributeSet.EMPTY) {
            return (MutableAttributeSet)set;
        }
        return this.createLargeAttributeSet(set);
    }
    
    @Override
    public String toString() {
        this.removeUnusedSets();
        String string = "";
        Iterator<StyleContext.SmallAttributeSet> iterator = this.attributesPool.keySet().iterator();
        while (iterator.hasNext()) {
            string = string + (Object)iterator.next() + "\n";
        }
        return string;
    }
    
    public void writeAttributes(ObjectOutputStream objectOutputStream, AttributeSet set) throws IOException {
        writeAttributeSet(objectOutputStream, set);
    }
    
    public void readAttributes(ObjectInputStream objectInputStream, MutableAttributeSet set) throws ClassNotFoundException, IOException {
        readAttributeSet(objectInputStream, set);
    }
    
    public static void writeAttributeSet(ObjectOutputStream objectOutputStream, AttributeSet set) throws IOException {
        objectOutputStream.writeInt(set.getAttributeCount());
        Enumeration<Object> attributeNames = set.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            if ((nextElement instanceof Serializable)) {
                objectOutputStream.writeObject(nextElement);
            }
            else {
                Object value = StyleContext.freezeKeyMap.get(nextElement);
                if (value == null) {
                    throw new NotSerializableException(nextElement.getClass().getName() + " is not serializable as a key in an AttributeSet");
                }
                objectOutputStream.writeObject(value);
            }
            Object attribute = set.getAttribute(nextElement);
            Object value2 = StyleContext.freezeKeyMap.get(attribute);
            if ((attribute instanceof Serializable)) {
                objectOutputStream.writeObject((value2 != null) ? value2 : attribute);
            }
            else {
                if (value2 == null) {
                    throw new NotSerializableException(attribute.getClass().getName() + " is not serializable as a value in an AttributeSet");
                }
                objectOutputStream.writeObject(value2);
            }
        }
    }
    
    public static void readAttributeSet(ObjectInputStream objectInputStream, MutableAttributeSet set) throws ClassNotFoundException, IOException {
        for (int int1 = objectInputStream.readInt(), i = 0; i < int1; ++i) {
            Object object = objectInputStream.readObject();
            Object object2 = objectInputStream.readObject();
            if (StyleContext.thawKeyMap != null) {
                Object value = StyleContext.thawKeyMap.get(object);
                if (value != null) {
                    object = value;
                }
                Object value2 = StyleContext.thawKeyMap.get(object2);
                if (value2 != null) {
                    object2 = value2;
                }
            }
            set.addAttribute(object, object2);
        }
    }
    
    public static void registerStaticAttributeKey(Object o) {
        String string = o.getClass().getName() + "." + o.toString();
        if (StyleContext.freezeKeyMap == null) {
            StyleContext.freezeKeyMap = new Hashtable();
            StyleContext.thawKeyMap = new Hashtable();
        }
        StyleContext.freezeKeyMap.put(o, string);
        StyleContext.thawKeyMap.put(string, o);
    }
    
    public static Object getStaticAttribute(Object key) {
        if (StyleContext.thawKeyMap == null || key == null) {
            return null;
        }
        return StyleContext.thawKeyMap.get(key);
    }
    
    public static Object getStaticAttributeKey(Object o) {
        return o.getClass().getName() + "." + o.toString();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        this.removeUnusedSets();
        objectOutputStream.defaultWriteObject();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        this.fontSearch = new StyleContext.FontKey(null, 0, 0);
        this.fontTable = new Hashtable();
        this.search = new SimpleAttributeSet();
        this.attributesPool = Collections.synchronizedMap(new WeakHashMap());
        objectInputStream.defaultReadObject();
    }
    
    static {
        try {
            for (int length = StyleConstants.keys.length, i = 0; i < length; ++i) {
                registerStaticAttributeKey(StyleConstants.keys[i]);
            }
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
    }
    
    public class SmallAttributeSet implements AttributeSet
    {
        Object[] attributes;
        AttributeSet resolveParent;
        
        public SmallAttributeSet(Object[] attributes) {
            this.attributes = attributes;
            this.updateResolveParent();
        }
        
        public SmallAttributeSet(AttributeSet set) {
            Object[] attributes = new Object[2 * set.getAttributeCount()];
            Enumeration<Object> attributeNames = set.getAttributeNames();
            int n = 0;
            while (attributeNames.hasMoreElements()) {
                attributes[n] = attributeNames.nextElement();
                attributes[n + 1] = set.getAttribute(attributes[n]);
                n += 2;
            }
            this.attributes = attributes;
            this.updateResolveParent();
        }
        
        private void updateResolveParent() {
            this.resolveParent = null;
            Object[] attributes = this.attributes;
            int i = 0;
            while (i < attributes.length) {
                if (attributes[i] == StyleConstants.ResolveAttribute) {
                    this.resolveParent = (AttributeSet)attributes[i + 1];
                    break;
                }
                i += 2;
            }
        }
        
        Object getLocalAttribute(Object o) {
            if (o == StyleConstants.ResolveAttribute) {
                return this.resolveParent;
            }
            Object[] attributes = this.attributes;
            int i = 0;
            while (i < attributes.length) {
                if (o.equals(attributes[i])) {
                    return attributes[i + 1];
                }
                i += 2;
            }
            return null;
        }
        
        @Override
        public String toString() {
            String str = "{";
            Object[] attributes = this.attributes;
            for (int i = 0; i < attributes.length; i += 2) {
                if ((attributes[i + 1] instanceof AttributeSet)) {
                    str = str + attributes[i] + "=AttributeSet,";
                }
                else {
                    str = str + attributes[i] + "=" + attributes[i + 1] + ",";
                }
            }
            return str + "}";
        }
        
        @Override
        public int hashCode() {
            int n = 0;
            Object[] attributes = this.attributes;
            for (int i = 1; i < attributes.length; i += 2) {
                n ^= attributes[i].hashCode();
            }
            return n;
        }
        
        @Override
        public boolean equals(Object o) {
            if ((o instanceof AttributeSet)) {
                AttributeSet set = (AttributeSet)o;
                return this.getAttributeCount() == set.getAttributeCount() && this.containsAttributes(set);
            }
            return false;
        }
        
        public Object clone() {
            return this;
        }
        
        @Override
        public int getAttributeCount() {
            return this.attributes.length / 2;
        }
        
        @Override
        public boolean isDefined(Object o) {
            Object[] attributes = this.attributes;
            int length = attributes.length;
            int i = 0;
            while (i < length) {
                if (o.equals(attributes[i])) {
                    return true;
                }
                i += 2;
            }
            return false;
        }
        
        @Override
        public boolean isEqual(AttributeSet set) {
            if ((set instanceof StyleContext.SmallAttributeSet)) {
                return set == this;
            }
            return this.getAttributeCount() == set.getAttributeCount() && this.containsAttributes(set);
        }
        
        @Override
        public AttributeSet copyAttributes() {
            return this;
        }
        
        @Override
        public Object getAttribute(Object o) {
            Object o2 = this.getLocalAttribute(o);
            if (o2 == null) {
                AttributeSet resolveParent = this.getResolveParent();
                if (resolveParent != null) {
                    o2 = resolveParent.getAttribute(o);
                }
            }
            return o2;
        }
        
        @Override
        public Enumeration<?> getAttributeNames() {
            return new StyleContext.KeyEnumeration(this.attributes);
        }
        
        @Override
        public boolean containsAttribute(Object o, Object o2) {
            return o2.equals(this.getAttribute(o));
        }
        
        @Override
        public boolean containsAttributes(AttributeSet set) {
            boolean equals = true;
            Object nextElement;
            for (Enumeration attributeNames = set.getAttributeNames(); equals && attributeNames.hasMoreElements(); equals = set.getAttribute(nextElement).equals(this.getAttribute(nextElement))) {
                nextElement = attributeNames.nextElement();
            }
            return equals;
        }
        
        @Override
        public AttributeSet getResolveParent() {
            return this.resolveParent;
        }
    }
    
    class KeyEnumeration implements Enumeration<Object>
    {
        Object[] attr;
        int i;
        
        KeyEnumeration(Object[] attr) {
            this.attr = attr;
            this.i = 0;
        }
        
        @Override
        public boolean hasMoreElements() {
            return this.i < this.attr.length;
        }
        
        @Override
        public Object nextElement() {
            if (this.i < this.attr.length) {
                Object o = this.attr[this.i];
                this.i += 2;
                return o;
            }
            throw new NoSuchElementException();
        }
    }
    
    class KeyBuilder
    {
        private Vector<Object> keys;
        private Vector<Object> data;
        
        KeyBuilder() {
            this.keys = new Vector();
            this.data = new Vector();
        }
        
        public void initialize(AttributeSet set) {
            if ((set instanceof StyleContext.SmallAttributeSet)) {
                this.initialize(((StyleContext.SmallAttributeSet)set).attributes);
            }
            else {
                this.keys.removeAllElements();
                this.data.removeAllElements();
                Enumeration<Object> attributeNames = set.getAttributeNames();
                while (attributeNames.hasMoreElements()) {
                    Object nextElement = attributeNames.nextElement();
                    this.addAttribute(nextElement, set.getAttribute(nextElement));
                }
            }
        }
        
        private void initialize(Object[] array) {
            this.keys.removeAllElements();
            this.data.removeAllElements();
            for (int length = array.length, i = 0; i < length; i += 2) {
                this.keys.addElement(array[i]);
                this.data.addElement(array[i + 1]);
            }
        }
        
        public Object[] createTable() {
            int size = this.keys.size();
            Object[] array = new Object[2 * size];
            for (int i = 0; i < size; ++i) {
                int n = 2 * i;
                array[n] = this.keys.elementAt(i);
                array[n + 1] = this.data.elementAt(i);
            }
            return array;
        }
        
        int getCount() {
            return this.keys.size();
        }
        
        public void addAttribute(Object obj, Object obj2) {
            this.keys.addElement(obj);
            this.data.addElement(obj2);
        }
        
        public void addAttributes(AttributeSet set) {
            if ((set instanceof StyleContext.SmallAttributeSet)) {
                Object[] attributes = ((StyleContext.SmallAttributeSet)set).attributes;
                for (int length = attributes.length, i = 0; i < length; i += 2) {
                    this.addAttribute(attributes[i], attributes[i + 1]);
                }
            }
            else {
                Enumeration<Object> attributeNames = set.getAttributeNames();
                while (attributeNames.hasMoreElements()) {
                    Object nextElement = attributeNames.nextElement();
                    this.addAttribute(nextElement, set.getAttribute(nextElement));
                }
            }
        }
        
        public void removeAttribute(Object obj) {
            int size = this.keys.size();
            int i = 0;
            while (i < size) {
                if (this.keys.elementAt(i).equals(obj)) {
                    this.keys.removeElementAt(i);
                    this.data.removeElementAt(i);
                    return;
                }
                ++i;
            }
        }
        
        public void removeAttributes(Enumeration enumeration) {
            while (enumeration.hasMoreElements()) {
                this.removeAttribute(enumeration.nextElement());
            }
        }
        
        public void removeAttributes(AttributeSet set) {
            Enumeration<Object> attributeNames = set.getAttributeNames();
            while (attributeNames.hasMoreElements()) {
                Object nextElement = attributeNames.nextElement();
                this.removeSearchAttribute(nextElement, set.getAttribute(nextElement));
            }
        }
        
        private void removeSearchAttribute(Object obj, Object obj2) {
            int size = this.keys.size();
            int i = 0;
            while (i < size) {
                if (this.keys.elementAt(i).equals(obj)) {
                    if (this.data.elementAt(i).equals(obj2)) {
                        this.keys.removeElementAt(i);
                        this.data.removeElementAt(i);
                    }
                    return;
                }
                else {
                    ++i;
                }
            }
        }
    }
    
    static class FontKey
    {
        private String family;
        private int style;
        private int size;
        
        public FontKey(String s, int n, int n2) {
            this.setValue(s, n, n2);
        }
        
        public void setValue(String s, int style, int size) {
            this.family = ((s != null) ? s.intern() : null);
            this.style = style;
            this.size = size;
        }
        
        @Override
        public int hashCode() {
            return ((this.family != null) ? this.family.hashCode() : 0) ^ this.style ^ this.size;
        }
        
        @Override
        public boolean equals(Object o) {
            if ((o instanceof StyleContext.FontKey)) {
                StyleContext.FontKey fontKey = (StyleContext.FontKey)o;
                return this.size == fontKey.size && this.style == fontKey.style && this.family == fontKey.family;
            }
            return false;
        }
    }
    
    public class NamedStyle implements Style, Serializable
    {
        protected EventListenerList listenerList;
        protected transient ChangeEvent changeEvent;
        private transient AttributeSet attributes;
        
        public NamedStyle(String name, Style resolveParent) {
            this.listenerList = new EventListenerList();
            this.changeEvent = null;
            this.attributes = StyleContext.this.getEmptySet();
            if (name != null) {
                this.setName(name);
            }
            if (resolveParent != null) {
                this.setResolveParent(resolveParent);
            }
        }
        
        public NamedStyle(StyleContext styleContext, Style style) {
            this(styleContext, null, style);
        }
        
        public NamedStyle() {
            this.listenerList = new EventListenerList();
            this.changeEvent = null;
            this.attributes = StyleContext.this.getEmptySet();
        }
        
        @Override
        public String toString() {
            return "NamedStyle:" + this.getName() + " " + this.attributes;
        }
        
        @Override
        public String getName() {
            if (this.isDefined(StyleConstants.NameAttribute)) {
                return this.getAttribute(StyleConstants.NameAttribute).toString();
            }
            return null;
        }
        
        public void setName(String s) {
            if (s != null) {
                this.addAttribute(StyleConstants.NameAttribute, s);
            }
        }
        
        @Override
        public void addChangeListener(ChangeListener l) {
            this.listenerList.add(ChangeListener.class, l);
        }
        
        @Override
        public void removeChangeListener(ChangeListener l) {
            this.listenerList.remove(ChangeListener.class, l);
        }
        
        public ChangeListener[] getChangeListeners() {
            return (ChangeListener[])this.listenerList.getListeners(ChangeListener.class);
        }
        
        protected void fireStateChanged() {
            Object[] listenerList = this.listenerList.getListenerList();
            for (int i = listenerList.length - 2; i >= 0; i -= 2) {
                if (listenerList[i] == ChangeListener.class) {
                    if (this.changeEvent == null) {
                        this.changeEvent = new ChangeEvent(this);
                    }
                    ((ChangeListener)listenerList[i + 1]).stateChanged(this.changeEvent);
                }
            }
        }
        
        public <T extends EventListener> T[] getListeners(Class<T> t) {
            return this.listenerList.getListeners(t);
        }
        
        @Override
        public int getAttributeCount() {
            return this.attributes.getAttributeCount();
        }
        
        @Override
        public boolean isDefined(Object o) {
            return this.attributes.isDefined(o);
        }
        
        @Override
        public boolean isEqual(AttributeSet set) {
            return this.attributes.isEqual(set);
        }
        
        @Override
        public AttributeSet copyAttributes() {
            StyleContext.NamedStyle namedStyle = new StyleContext.NamedStyle();
            namedStyle.attributes = this.attributes.copyAttributes();
            return namedStyle;
        }
        
        @Override
        public Object getAttribute(Object o) {
            return this.attributes.getAttribute(o);
        }
        
        @Override
        public Enumeration<?> getAttributeNames() {
            return this.attributes.getAttributeNames();
        }
        
        @Override
        public boolean containsAttribute(Object o, Object o2) {
            return this.attributes.containsAttribute(o, o2);
        }
        
        @Override
        public boolean containsAttributes(AttributeSet set) {
            return this.attributes.containsAttributes(set);
        }
        
        @Override
        public AttributeSet getResolveParent() {
            return this.attributes.getResolveParent();
        }
        
        @Override
        public void addAttribute(Object o, Object o2) {
            this.attributes = StyleContext.this.addAttribute(this.attributes, o, o2);
            this.fireStateChanged();
        }
        
        @Override
        public void addAttributes(AttributeSet set) {
            this.attributes = StyleContext.this.addAttributes(this.attributes, set);
            this.fireStateChanged();
        }
        
        @Override
        public void removeAttribute(Object o) {
            this.attributes = StyleContext.this.removeAttribute(this.attributes, o);
            this.fireStateChanged();
        }
        
        @Override
        public void removeAttributes(Enumeration<?> enumeration) {
            this.attributes = StyleContext.this.removeAttributes(this.attributes, enumeration);
            this.fireStateChanged();
        }
        
        @Override
        public void removeAttributes(AttributeSet set) {
            StyleContext this$0 = StyleContext.this;
            if (set == this) {
                this.attributes = this$0.getEmptySet();
            }
            else {
                this.attributes = this$0.removeAttributes(this.attributes, set);
            }
            this.fireStateChanged();
        }
        
        @Override
        public void setResolveParent(AttributeSet set) {
            if (set != null) {
                this.addAttribute(StyleConstants.ResolveAttribute, set);
            }
            else {
                this.removeAttribute(StyleConstants.ResolveAttribute);
            }
        }
        
        private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
            objectOutputStream.defaultWriteObject();
            StyleContext.writeAttributeSet(objectOutputStream, this.attributes);
        }
        
        private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
            objectInputStream.defaultReadObject();
            this.attributes = SimpleAttributeSet.EMPTY;
            StyleContext.readAttributeSet(objectInputStream, this);
        }
    }
}
